@@ -1,6 +1,4 @@
 
-# this script's functions plot a given query spectrum against a given reference spectrum before and after spectrum preprocessing transformations
-
 from .processing import *
 from .similarity_measures import *
 import pandas as pd
@@ -36,7 +34,6 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
     --output_path: path to output PDF file containing the plots of the spectra before and after preprocessing transformations. If no argument is passed, then the plots will be saved to the PDF ./spectrum1_{spectrum_ID1}_spectrum2_{spectrum_ID2}_plot.pdf in the current working directory.
     '''
 
-    # load query and reference libraries
     if query_data is None:
         print('\nError: No argument passed to the mandatory query_data. Please pass the path to the CSV file of the query data.')
         sys.exit()
@@ -45,7 +42,7 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
         extension = extension[(len(extension)-1)]
         if extension == 'mgf' or extension == 'MGF' or extension == 'mzML' or extension == 'mzml' or extension == 'MZML' or extension == 'cdf' or extension == 'CDF':
             output_path_tmp = query_data[:-3] + 'csv'
-            build_library_from_raw_data(input_path=query_data, output_path=output_path_tmp, is_reference=False)
+            build_library_from_raw_data(input_path=query_data, output_path=output_path_tmp, is_reference=True)
             df_query = pd.read_csv(output_path_tmp)
         if extension == 'csv' or extension == 'CSV':
             df_query = pd.read_csv(query_data)
@@ -68,7 +65,6 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
         unique_reference_ids = [str(tmp) for tmp in unique_reference_ids]
 
 
-    ##### process input parameters and ensure they are in a valid format #####
     if spectrum_ID1 is not None:
         spectrum_ID1 = str(spectrum_ID1)
     else:
@@ -96,8 +92,8 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
         print(f'Error: spectrum_preprocessing_order must contain only \'C\', \'F\', \'M\', \'N\', \'L\', \'W\'.')
         sys.exit()
 
-    if similarity_measure not in ['cosine','shannon','renyi','tsallis','mixture','jaccard','dice','3w_jaccard','sokal_sneath','binary_cosine','mountford','mcconnaughey','driver_kroeber','simpson','braun_banquet','fager_mcgowan','kulczynski','interection','hamming','hellinger']:
-        print('\nError: similarity_measure must be either cosine, shannon, renyi, tsallis, mixture, jaccard, dice, 3w_jaccard, sokal_sneath, binary_cosine, mountford, mcconnaughey, driver_kroeber, simpson, braun_banquet, fager_mcgowan, kulczynski, interection, hamming, or hellinger.')
+    if similarity_measure not in ['cosine','shannon','renyi','tsallis','mixture','jaccard','dice','3w_jaccard','sokal_sneath','binary_cosine','mountford','mcconnaughey','driver_kroeber','simpson','braun_banquet','fager_mcgowan','kulczynski','intersection','hamming','hellinger']:
+        print('\nError: similarity_measure must be either cosine, shannon, renyi, tsallis, mixture, jaccard, dice, 3w_jaccard, sokal_sneath, binary_cosine, mountford, mcconnaughey, driver_kroeber, simpson, braun_banquet, fager_mcgowan, kulczynski, intersection, hamming, or hellinger.')
         sys.exit()
 
     if isinstance(int_min,int) is True:
@@ -190,7 +186,6 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
     q_spec_pre_trans[:,1] = q_spec_pre_trans[:,1].astype(float)
     r_spec_pre_trans[:,1] = r_spec_pre_trans[:,1].astype(float)
 
-    # apply transformation to y-axis if relevant
     if y_axis_transformation == 'normalized':
         q_spec_pre_trans[:,1] = q_spec_pre_trans[:,1] / np.max(q_spec_pre_trans[:,1])
         r_spec_pre_trans[:,1] = r_spec_pre_trans[:,1] / np.max(r_spec_pre_trans[:,1])
@@ -206,10 +201,8 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
     else:
         ylab = 'Raw Intensity'
 
-    # create the figure
     fig, axes = plt.subplots(nrows=2, ncols=1)
 
-    # plot the untransformed spectra
     plt.subplot(2,1,1)
     plt.vlines(x=q_spec_pre_trans[:,0], ymin=[0]*q_spec_pre_trans.shape[0], ymax=q_spec_pre_trans[:,1], linewidth=3, color='blue', label=f'Spectrum ID 1: {spectrum_ID1}')
     plt.vlines(x=r_spec_pre_trans[:,0], ymin=[0]*r_spec_pre_trans.shape[0], ymax=-r_spec_pre_trans[:,1], linewidth=3, color='red', label=f'Spectrum ID 2: {spectrum_ID2}')
@@ -219,7 +212,6 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
     plt.yticks(fontsize=7)
     plt.title('Untransformed Spectra', fontsize=10)
 
-    # get the ranges of m/z and intensity values to display at the bottom of the two plots
     mz_min_tmp_q = round(q_spec[:,0].min(),1)
     mz_min_tmp_r = round(r_spec[:,0].min(),1)
     int_min_tmp_q = round(q_spec[:,1].min(),1)
@@ -233,51 +225,47 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
     int_min_tmp = min([int_min_tmp_q,int_min_tmp_r])
     int_max_tmp = max([int_max_tmp_q,int_max_tmp_r])
 
-    # perform the spectrum preprocessing transformations in the order specified
     is_matched = False
     for transformation in spectrum_preprocessing_order:
-        if transformation == 'C' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1: # centroiding
+        if transformation == 'C' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
             q_spec = centroid_spectrum(q_spec, window_size=window_size_centroiding) 
             r_spec = centroid_spectrum(r_spec, window_size=window_size_centroiding) 
-        if transformation == 'M' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1: # matching
+        if transformation == 'M' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
             m_spec = match_peaks_in_spectra(spec_a=q_spec, spec_b=r_spec, window_size=window_size_matching)
             q_spec = m_spec[:,0:2]
             r_spec = m_spec[:,[0,2]]
             is_matched = True
-        if transformation == 'W' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1: # weight factor transformation
+        if transformation == 'W' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
             q_spec[:,1] = wf_transform(q_spec[:,0], q_spec[:,1], wf_mz, wf_intensity)
             r_spec[:,1] = wf_transform(r_spec[:,0], r_spec[:,1], wf_mz, wf_intensity)
-        if transformation == 'L' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1: # low-entropy transformation
+        if transformation == 'L' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
             q_spec[:,1] = LE_transform(q_spec[:,1], LET_threshold, normalization_method=normalization_method)
             r_spec[:,1] = LE_transform(r_spec[:,1], LET_threshold, normalization_method=normalization_method)
-        if transformation == 'N' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1: # noise removal
+        if transformation == 'N' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
             q_spec = remove_noise(q_spec, nr = noise_threshold)
-            r_spec = remove_noise(r_spec, nr = noise_threshold)
-        if transformation == 'F' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1: # filtering
+            if high_quality_reference_library == False or high_quality_reference_library == 'False':
+                r_spec = remove_noise(r_spec, nr = noise_threshold)
+        if transformation == 'F' and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
             q_spec = filter_spec_lcms(q_spec, mz_min = mz_min, mz_max = mz_max, int_min = int_min, int_max = int_max, is_matched = is_matched)
-            r_spec = filter_spec_lcms(r_spec, mz_min = mz_min, mz_max = mz_max, int_min = int_min, int_max = int_max, is_matched = is_matched)
+            if high_quality_reference_library == False or high_quality_reference_library == 'False':
+                r_spec = filter_spec_lcms(r_spec, mz_min = mz_min, mz_max = mz_max, int_min = int_min, int_max = int_max, is_matched = is_matched)
 
-    # intensities of query and reference library
     q_ints = q_spec[:,1]
     r_ints = r_spec[:,1]
 
-    # if there is at least one non-zero intensity ion fragment in either spectra, compute their similarity
     if np.sum(q_ints) != 0 and np.sum(r_ints) != 0 and q_spec.shape[0] > 1 and r_spec.shape[1] > 1:
         similarity_score = get_similarity(similarity_measure, q_ints, r_ints, weights, entropy_dimension)
     else:
         similarity_score = 0
 
-    # plot the transformed spectra
     plt.subplot(2,1,2)
 
-    # display warning message if either spectra are empty or have no non-zero intensity ion fragments
     if q_spec.shape[0] > 1:
         if np.max(q_spec[:,1]) == 0 or np.max(r_spec[:,1]) == 0:
             plt.text(0.5, 0.5, 'The query and/or reference spectrum has no non-zero intensities after transformations.\n Change transformation parameters.', ha='center', va='center', fontsize=7, color='black')
             plt.xticks([])
             plt.yticks([])
         else:
-            # apply transformation to y-axis if relevant
             if y_axis_transformation == 'normalized':
                 q_spec[:,1] = q_spec[:,1] / np.max(q_spec[:,1])
                 r_spec[:,1] = r_spec[:,1] / np.max(r_spec[:,1])
@@ -305,12 +293,15 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
         plt.yticks([])
 
 
+    print('\n\n\n')
+    print(high_quality_reference_library)
+    print('\n\n\n')
     plt.subplots_adjust(top=0.8, hspace=0.92, bottom=0.3)
     plt.figlegend(loc = 'upper center')
     fig.text(0.05, 0.18, f'Similarity Measure: {similarity_measure.capitalize()}', fontsize=7)
     fig.text(0.05, 0.15, f'Similarity Score: {round(similarity_score,4)}', fontsize=7)
-    fig.text(0.05, 0.12, f'Spectrum Preprocessing Order: {''.join(spectrum_preprocessing_order)}', fontsize=7)
-    fig.text(0.05, 0.09, f'High Quality Reference Library: {high_quality_reference_library}', fontsize=7)
+    fig.text(0.05, 0.12, f"Spectrum Preprocessing Order: {''.join(spectrum_preprocessing_order)}", fontsize=7)
+    fig.text(0.05, 0.09, f'High Quality Reference Library: {str(high_quality_reference_library)}', fontsize=7)
     fig.text(0.05, 0.06, f'Window Size (Centroiding): {window_size_centroiding}', fontsize=7)
     fig.text(0.05, 0.03, f'Window Size (Matching): {window_size_matching}', fontsize=7)
     fig.text(0.45, 0.18, f'Raw-Scale M/Z Range: [{mz_min_tmp},{mz_max_tmp}]', fontsize=7)
@@ -318,6 +309,9 @@ def generate_plots_on_HRMS_data(query_data=None, reference_data=None, spectrum_I
     fig.text(0.45, 0.12, f'Noise Threshold: {noise_threshold}', fontsize=7)
     fig.text(0.45, 0.09, f'Weight Factors (m/z,intensity): ({wf_mz},{wf_intensity})', fontsize=7)
     fig.text(0.45, 0.06, f'Low-Entropy Threshold: {LET_threshold}', fontsize=7)
+    if similarity_measure == 'mixture':
+        fig.text(0.45, 0.03, f'Weights for mixture similarity: {weights}', fontsize=7)
+
     plt.savefig(output_path, format='pdf')
 
     if return_plot == True:
@@ -349,7 +343,6 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
     --output_path: path to output PDF file containing the plots of the spectra before and after preprocessing transformations. If no argument is passed, then the plots will be saved to the PDF ./spectrum1_{spectrum_ID1}_spectrum2_{spectrum_ID2}_plot.pdf in the current working directory.
     '''
 
-    # load query and reference libraries
     if query_data is None:
         print('\nError: No argument passed to the mandatory query_data. Please pass the path to the CSV file of the query data.')
         sys.exit()
@@ -379,7 +372,6 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
             unique_reference_ids = df_reference.iloc[:,0].unique()
 
 
-    ##### process input parameters and ensure they are in a valid format #####
     if spectrum_ID1 is not None:
         spectrum_ID1 = str(spectrum_ID1)
     else:
@@ -400,8 +392,8 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
         print(f'Error: spectrum_preprocessing_order must contain only \'F\', \'N\', \'W\', \'L\'.')
         sys.exit()
 
-    if similarity_measure not in ['cosine','shannon','renyi','tsallis','mixture','jaccard','dice','3w_jaccard','sokal_sneath','binary_cosine','mountford','mcconnaughey','driver_kroeber','simpson','braun_banquet','fager_mcgowan','kulczynski','interection','hamming','hellinger']:
-        print('\nError: similarity_measure must be either cosine, shannon, renyi, tsallis, mixture, jaccard, dice, 3w_jaccard, sokal_sneath, binary_cosine, mountford, mcconnaughey, driver_kroeber, simpson, braun_banquet, fager_mcgowan, kulczynski, interection, hamming, or hellinger.')
+    if similarity_measure not in ['cosine','shannon','renyi','tsallis','mixture','jaccard','dice','3w_jaccard','sokal_sneath','binary_cosine','mountford','mcconnaughey','driver_kroeber','simpson','braun_banquet','fager_mcgowan','kulczynski','intersection','hamming','hellinger']:
+        print('\nError: similarity_measure must be either cosine, shannon, renyi, tsallis, mixture, jaccard, dice, 3w_jaccard, sokal_sneath, binary_cosine, mountford, mcconnaughey, driver_kroeber, simpson, braun_banquet, fager_mcgowan, kulczynski, intersection, hamming, or hellinger.')
         sys.exit()
 
     if isinstance(int_min,int) is True:
@@ -454,12 +446,10 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
         print(f'Warning: plots will be saved to the PDF ./spectrum1_{spectrum_ID1}_spectrum2_{spectrum_ID2}_plot.pdf in the current working directory.')
         output_path = f'{Path.cwd()}/spectrum1_{spectrum_ID1}_spectrum2_{spectrum_ID2}.pdf'
 
-    # get m/z values
     min_mz = np.min([np.min(df_query.iloc[:,1]), np.min(df_reference.iloc[:,1])])
     max_mz = np.max([np.max(df_query.iloc[:,1]), np.max(df_reference.iloc[:,1])])
     mzs = np.linspace(min_mz,max_mz,(max_mz-min_mz+1))
 
-    # get unique query/reference library IDs; each query/reference ID corresponds to exactly one query/reference mass spectrum
     unique_query_ids = df_query.iloc[:,0].unique().tolist()
     unique_reference_ids = df_reference.iloc[:,0].unique().tolist()
     unique_query_ids = [str(ID) for ID in unique_query_ids]
@@ -491,7 +481,6 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
     q_spec = convert_spec(q_spec,mzs)
     r_spec = convert_spec(r_spec,mzs)
 
-    # get the ranges of m/z and intensity values to display at the bottom of the two plots
     int_min_tmp_q = min(q_spec[q_spec[:,1].nonzero(),1][0])
     int_min_tmp_r = min(r_spec[r_spec[:,1].nonzero(),1][0])
     int_max_tmp_q = max(q_spec[q_spec[:,1].nonzero(),1][0])
@@ -499,13 +488,10 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
     int_min_tmp = int(min([int_min_tmp_q,int_min_tmp_r]))
     int_max_tmp = int(max([int_max_tmp_q,int_max_tmp_r]))
     
-    # create the figure
     fig, axes = plt.subplots(nrows=2, ncols=1)
 
-    # plot the untransformed spectra
     plt.subplot(2,1,1)
 
-    # display warning message if either spectra have no non-zero ion fragments
     if np.max(q_spec[:,1]) == 0 or np.max(r_spec[:,1]) == 0:
         plt.text(0.5, 0.5, 'The query and/or reference spectrum has no non-zero intensities after transformations.\n Change transformation parameters.', ha='center', va='center', fontsize=7, color='black')
         plt.xticks([])
@@ -516,7 +502,6 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
         q_spec_pre_trans[:,1] = q_spec_pre_trans[:,1].astype(float)
         r_spec_pre_trans[:,1] = r_spec_pre_trans[:,1].astype(float)
 
-        # apply transformation to y-axis if relevant
         if y_axis_transformation == 'normalized':
             q_spec_pre_trans[:,1] = q_spec_pre_trans[:,1] / np.max(q_spec_pre_trans[:,1])
             r_spec_pre_trans[:,1] = r_spec_pre_trans[:,1] / np.max(r_spec_pre_trans[:,1])
@@ -540,32 +525,29 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
         plt.title('Untransformed Query and Reference Spectra', fontsize=10)
 
     for transformation in spectrum_preprocessing_order:
-        if transformation == 'W': # weight factor transformation
+        if transformation == 'W':
             q_spec[:,1] = wf_transform(q_spec[:,0], q_spec[:,1], wf_mz, wf_intensity)
             r_spec[:,1] = wf_transform(r_spec[:,0], r_spec[:,1], wf_mz, wf_intensity)
-        if transformation == 'L': # low-entropy transformation
+        if transformation == 'L':
             q_spec[:,1] = LE_transform(q_spec[:,1], LET_threshold, normalization_method)
             r_spec[:,1] = LE_transform(r_spec[:,1], LET_threshold, normalization_method)
-        if transformation == 'N': # noise removal
+        if transformation == 'N':
             q_spec = remove_noise(q_spec, nr = noise_threshold)
-            if high_quality_reference_library == False:
+            if high_quality_reference_library == False or high_quality_reference_library == 'False':
                 r_spec = remove_noise(r_spec, nr = noise_threshold)
-        if transformation == 'F': # filtering with respect to mz and/or intensity
+        if transformation == 'F':
             q_spec = filter_spec_gcms(q_spec, mz_min = mz_min, mz_max = mz_max, int_min = int_min, int_max = int_max)
-            if high_quality_reference_library == False:
+            if high_quality_reference_library == False or high_quality_reference_library == 'False':
                 r_spec = filter_spec_gcms(r_spec, mz_min = mz_min, mz_max = mz_max, int_min = int_min, int_max = int_max)
 
-    # compute similarity score; if the spectra contain at most one point, their similarity is considered to be 0
     if q_spec.shape[0] > 1:
         similarity_score = get_similarity(similarity_measure, q_spec[:,1], r_spec[:,1], weights, entropy_dimension)
     else:
         similarity_score = 0
 
  
-    # plot the transformed spectra
     plt.subplot(2,1,2)
 
-    # display warning message if either spectra are empty or have no non-zero intensity ion fragments
     if q_spec.shape[0] == 0 or r_spec.shape[0] == 0:
         plt.text(0.5, 0.5, 'The query and/or reference spectrum has no ion fragments left after transformations.\n Change transformation parameters.', ha='center', va='center', fontsize=7, color='black')
         plt.xticks([])
@@ -575,7 +557,6 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
         plt.xticks([])
         plt.yticks([])
     else:
-        # apply transformation to y-axis if relevant
         if y_axis_transformation == 'normalized':
             q_spec[:,1] = q_spec[:,1] / np.max(q_spec[:,1])
             r_spec[:,1] = r_spec[:,1] / np.max(r_spec[:,1])
@@ -599,18 +580,19 @@ def generate_plots_on_NRMS_data(query_data=None, reference_data=None, spectrum_I
         plt.title(f'Transformed Query and Reference Spectra', fontsize=10)
 
 
-    #plt.subplots_adjust(top = 0.8, hspace = 0.7)
     plt.subplots_adjust(top=0.8, hspace=0.92, bottom=0.3)
     plt.figlegend(loc = 'upper center')
     fig.text(0.05, 0.15, f'Similarity Measure: {similarity_measure.capitalize()}', fontsize=7)
     fig.text(0.05, 0.12, f'Similarity Score: {round(similarity_score,4)}', fontsize=7)
-    fig.text(0.05, 0.09, f'Spectrum Preprocessing Order: {''.join(spectrum_preprocessing_order)}', fontsize=7)
-    fig.text(0.05, 0.06, f'High Quality Reference Library: {high_quality_reference_library}', fontsize=7)
+    fig.text(0.05, 0.09, f"Spectrum Preprocessing Order: {''.join(spectrum_preprocessing_order)}", fontsize=7)
+    fig.text(0.05, 0.06, f'High Quality Reference Library: {str(high_quality_reference_library)}', fontsize=7)
     fig.text(0.05, 0.03, f'Raw-Scale M/Z Range: [{min_mz},{max_mz}]', fontsize=7)
     fig.text(0.45, 0.15, f'Raw-Scale Intensity Range: [{int_min_tmp},{int_max_tmp}]', fontsize=7)
     fig.text(0.45, 0.12, f'Noise Threshold: {noise_threshold}', fontsize=7)
     fig.text(0.45, 0.09, f'Weight Factors (m/z,intensity): ({wf_mz},{wf_intensity})', fontsize=7)
     fig.text(0.45, 0.06, f'Low-Entropy Threshold: {LET_threshold}', fontsize=7)
+    if similarity_measure=='mixture':
+        fig.text(0.45, 0.03, f'Weights for mixture similarity: {weights}', fontsize=7)
     plt.savefig(output_path, format='pdf')
 
     if return_plot == True:
