@@ -1,5 +1,5 @@
 
-from pycompound_fy7392.build_library import build_library_from_raw_data
+from pycompound.build_library import build_library_from_raw_data
 from pathlib import Path
 import os
 
